SystemOrganization addCategory: #'AST-Interpreter-Test'!

TestCase subclass: #ASTCacheTest
	instanceVariableNames: 'cache compiledMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter-Test'!

!ASTCacheTest methodsFor: 'running' stamp: 'CamilloBruni 2/17/2012 14:29'!
setUp
	ASTCache reset.
	compiledMethod := ASTCacheTest >> #setUp.! !

!ASTCacheTest methodsFor: 'tests' stamp: 'CamilloBruni 2/17/2012 14:26'!
testInCache
	self deny: (ASTCache default includesKey: compiledMethod).
	
	compiledMethod ast.
	
	self assert: (ASTCache default includesKey: compiledMethod).! !

!ASTCacheTest methodsFor: 'tests' stamp: 'CamilloBruni 2/27/2012 23:16'!
testRecompile
	compiledMethod ast.
	
	self assert: (ASTCache default includesKey: compiledMethod).
	
	"recompile the test method and avoid referring to the old method"
	compiledMethod := compiledMethod recompile.
	"trigger garbage collection to free the ASTCache"
	Smalltalk garbageCollect.
	
	self deny: (ASTCache default includesKey: compiledMethod).! !

!ASTCacheTest methodsFor: 'tests' stamp: 'CamilloBruni 2/17/2012 14:27'!
testReset
	compiledMethod ast.
	
	self assert: (ASTCache default includesKey: compiledMethod).
	
	ASTCache reset.
	
	self deny: (ASTCache default includesKey: compiledMethod).! !

TestCase subclass: #ASTInterpreterTest
	instanceVariableNames: 'interpreter context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter-Test'!

!ASTInterpreterTest class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 19:56'!
testMethodArguments: anObject
	^ [ anObject ] value! !

!ASTInterpreterTest methodsFor: 'helper' stamp: 'CamilloBruni 12/5/2011 19:01'!
compile: aString
	| ast |
	ast := RBParser parseExpression: aString.
	ast annotateInClass: ProtoObject.
	^ ast! !

!ASTInterpreterTest methodsFor: 'helper' stamp: 'CamilloBruni 12/12/2011 15:03'!
interpret: aString
	| result |
	
	"context homeContext: thisContext".
	interpreter resetContext: context.
	result := interpreter interpret: (self compile: aString).
	
	self assert: interpreter context = context.
	^ result! !

!ASTInterpreterTest methodsFor: 'helper' stamp: 'CamilloBruni 12/9/2011 22:21'!
setUp
	"for now we use an ast cache to speed up the ast lookup"
	ASTCache initialize.
	interpreter := ASTInterpreter new.
	context := AIRootContext new.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 19:53'!
testBasicCode
	self assert: (self interpret: '[ true ifTrue: [ nil ] ifFalse: [ 1 ]] value') = nil.
	self assert: (self interpret: '[ false ifTrue: [ nil ] ifFalse: [ 1 ]] value') = 1.
	
	self assert: (self interpret: '[ |a| a :=1. [ a < 10 ] whileTrue: [ a := a + 1]. a] value') = 10.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 10/5/2011 17:39'!
testBinarySend
	self assert: (self interpret: '1 + 2') = 3.
	self assert: (self interpret: '1 < 2') = true.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 18:26'!
testBlock
	self assert: ((self interpret: '[ 1 ]') isKindOf: ASTBlockClosure).
	
	self assert: (self interpret: '[ 1 ] value') = 1.
	self assert: (self interpret: '[ 1 + 2 ] value') = 3.
	
	self assert: (self interpret: 'true ifTrue: [ 1 ] ifFalse: [ 0 ]') = 1.
	self assert: (self interpret: 'false ifTrue: [ 1 ] ifFalse: [ 0 ]') = 0.
	! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 18:10'!
testBlockArgument

	self assert: (self interpret: '[ :a| a ] value: 1') = 1.
	self assert: (self interpret: '[ :a| a + 3 ] value: 3 ') = 6.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 15:31'!
testBlockVar

	self assert: (self interpret: '[ |a| a := 1. a ] value') = 1.
	self assert: (self interpret: '[ |a| a := 1 + 2. a + 3 ] value') = 6.
	
	self assert: (self interpret: '[ |a| [ a := 1 ] value ] value') = 1.
	self assert: (self interpret: '[ |a| a := 1. [ a := 2 ] value ] value') = 2.
	
	self assert: (self interpret: '[ |a| a := 1. [ a := a + 1 ] value ] value') = 2.
	
	self assert: (self interpret: '[ :b ||a| a := 1. [ :c| a := a + 1 + c ] value: b ] value: 3') = 5.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 09:56'!
testConstant
	self assert: (self interpret: '1') = 1.
	self assert: (self interpret: 'nil') = nil.
	self assert: (self interpret: 'false') = false.
	self assert: (self interpret: '''asdf''') = 'asdf'.
	self assert: (self interpret: '#symbol') = #symbol.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 15:00'!
testDNU
	| error |
	
	[ self interpret: '1 aMessageNotUnderstoodBySmallInteger' ]
		on: InterpretationError
		do: [ :err| error := err ].
	
	self assert: error cause message selector = #aMessageNotUnderstoodBySmallInteger.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/13/2011 16:09'!
testEnsure

	self assert: (self interpret: '[ ] ensure: [ 2 ].') = nil.
	self assert: (self interpret: '[ 1 ] ensure: [ 2 ].') = 1.
	self assert: (self interpret: '[ 1 ] ensure: [ 2 ]. 3') = 3.
	
	self assert: (self interpret: '[ 1 ] ensure: [ ^ 2 ]. 3') = 2.
	self assert: (self interpret: '[ ^ 1 ] ensure: [ ^ 2 ]. 3') = 2.
	
"	self
		should: (self interpret: '[ Error signal ] ensure: [ ^ 2 ]. 3')
		raise: InterpretationError."! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 16:35'!
testException
	[ (self interpret: 'Error signal') ]
		on: InterpretationError
		do: [ :err| ^ self ].
	self fail.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 16:33'!
testExceptionHandling

	self assert: (self interpret: '[ ] on: Error do: [ :err| false ].') = nil.
	self assert: (self interpret: '[ ] on: Error do: [ :err| false ]. true') = true.
	self assert: (self interpret: '[ ] on: Error do: [ :err| ^ false ]. true') = true.
	
	self assert: (self interpret: '[ Error signal ] on: Error do: [ :err| false ]') = false.
	self assert: (self interpret: '[ Error signal ] on: Error do: [ :err| false ]. true') = true.
	self assert: (self interpret: '[ Error signal ] on: Error do: [ :err| ^ false ]. true') = false.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 14:56'!
testGlobals
	self assert: (self interpret: 'Array') = Array! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 18:19'!
testInstanceCreation
	self assert: ((self interpret: 'Array new') isKindOf: Array).
	self assert: ((self interpret: 'Error new') isKindOf: Error).
	self assert: ((self interpret: 'MessageNotUnderstood new') isKindOf: MessageNotUnderstood)! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 19:55'!
testMethodArguments
	
	self assert: (self interpret: 'ASTInterpreterTest testMethodArguments: 1') = 1.
	! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 18:21'!
testNonLocalReturn

	self assert: (self interpret: 'false ifTrue: [ ^ 1 ]. ^ 1 + 1') = 2.
	self assert: (self interpret: 'true ifTrue: [ ^ 1 ]. ^ 1 + 1') = 1.
	
	self assert: (self interpret: 'false ifTrue: [ ^ 1 ] ifFalse: [^ 2]. 1 + 1 + 1') = 2.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 00:21'!
testPrimitive
	self assert: (self interpret: '1 + 1') = 2. "simple"
	self assert: (self interpret: '1 + 1.5') = 2.5. "with fallback"! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 16:39'!
testSend
	self assert: (self interpret: '#(1 2) at: 1') = 1.
	self assert: (self interpret: '1 + 2') = 3.
	"this is for future use :)"
	self assert: (self interpret: '123 asString') = '123'.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 11:24'!
testUnarySend
	self assert: (self interpret: '1 asInteger') = 1.
	self assert: (self interpret: '1 class') = SmallInteger.! !
